@@ -90,10 +90,9 @@ pub trait CRUDTable: Send + Sync + Serialize + DeserializeOwned {
         let source = m.get(column);
         match source {
             Some(s) => {
-                *data=s(&data);
-            },
-            _ => {
+                *data = s(&data);
             }
+            _ => {}
         }
     }
 
@@ -122,8 +121,8 @@ pub trait CRUDTable: Send + Sync + Serialize + DeserializeOwned {
             let v = map.get(column).unwrap_or(&serde_json::Value::Null);
             //cast convert
             column_sql = column_sql + column + ",";
-            let mut data =String::new();
-            db_type.stmt_convert(*index,&mut data);
+            let mut data = String::new();
+            db_type.stmt_convert(*index, &mut data);
             Self::do_format_column(db_type, &column, &mut data);
             value_sql = value_sql
                 + data.as_str()
@@ -223,13 +222,17 @@ pub trait CRUD {
     ) -> Result<DBExecResult>
         where
             T: CRUDTable;
+
     async fn save<T>(&self, table: &T) -> Result<DBExecResult>
         where
             T: CRUDTable;
+
     async fn save_batch<T>(&self, tables: &[T]) -> Result<DBExecResult>
         where
             T: CRUDTable;
 
+    /// save_batch_slice
+    /// batch save each slice_len every time
     async fn save_batch_slice<T>(
         &self,
         tables: &[T],
@@ -241,17 +244,24 @@ pub trait CRUD {
     async fn remove_by_wrapper<T>(&self, w: &Wrapper) -> Result<u64>
         where
             T: CRUDTable;
+
+    /// remove_by_column
+    /// column_value,column's value
     async fn remove_by_column<T, C>(&self, column: &str, column_value: &C) -> Result<u64> where T: CRUDTable, C: Serialize + Send + Sync;
 
+    /// remove_batch_by_column
+    /// column_values,column's value
     async fn remove_batch_by_column<T, C>(&self, column: &str, column_values: &[C]) -> Result<u64>
         where
             T: CRUDTable, C: Serialize + Send + Sync;
 
+    /// update_by_wrapper
+    /// skips: use &["id","null"] will skip id column and null value param
     async fn update_by_wrapper<T>(
         &self,
         table: &mut T,
         w: &Wrapper,
-        update_null_value: bool,
+        skips: &[&str],
     ) -> Result<u64>
         where
             T: CRUDTable;
@@ -482,8 +492,8 @@ pub trait CRUDMut: ExecutorMut {
     {
         let mut sql = String::new();
         let driver_type = &self.driver_type()?;
-        let mut data =String::new();
-        driver_type.stmt_convert(0,&mut data);
+        let mut data = String::new();
+        driver_type.stmt_convert(0, &mut data);
         T::do_format_column(&driver_type, column, &mut data);
         if self.get_rbatis().logic_plugin.is_some() && T::is_use_plugin(self.get_rbatis().logic_plugin.as_ref().unwrap().name()) {
             sql = self.get_rbatis().logic_plugin.as_ref().unwrap().create_remove_sql(
@@ -534,14 +544,13 @@ pub trait CRUDMut: ExecutorMut {
         return self.remove_by_wrapper::<T>(&w).await;
     }
 
-    /// update arg by wrapper
-    /// skips:  vec!["null"] ->  will skip  null fields
-    /// skips:  vec!["column"] ->  will skip  column fields
+    /// update_by_wrapper
+    /// skips: use &["id","null"] will skip id column and null value param
     async fn update_by_wrapper<T>(
         &mut self,
         table: &mut T,
         w: &Wrapper,
-        skips: Vec<&str>,
+        skips: &[&str],
     ) -> Result<u64>
         where
             T: CRUDTable,
@@ -564,7 +573,7 @@ pub trait CRUDMut: ExecutorMut {
         let null = serde_json::Value::Null;
         let mut sets = String::new();
         let mut skip_null_value = false;
-        for x in &skips {
+        for x in skips {
             if "null".eq(*x) {
                 skip_null_value = true;
             }
@@ -572,19 +581,23 @@ pub trait CRUDMut: ExecutorMut {
 
         for column in columns_vec {
             //filter
-            for x in &skips {
+            let mut is_continue = false;
+            for x in skips {
                 if column.eq(*x) {
-                    continue;
+                    is_continue = true;
                 }
             }
+            if is_continue {
+                continue;
+            }
             let mut v = map.get(column).unwrap_or_else(|| &null).clone();
             //filter null
             if skip_null_value && v.is_null() {
                 continue;
             }
 
-            let mut data=String::new();
-            driver_type.stmt_convert(args.len(),&mut data);
+            let mut data = String::new();
+            driver_type.stmt_convert(args.len(), &mut data);
             T::do_format_column(
                 &driver_type,
                 &column,
@@ -666,6 +679,7 @@ pub trait CRUDMut: ExecutorMut {
     }
 
     /// update database record by id
+    /// update sql will be skip null value and id column
     async fn update_by_column<T>(&mut self, column: &str, table: &mut T) -> Result<u64>
         where
             T: CRUDTable
@@ -678,7 +692,7 @@ pub trait CRUDMut: ExecutorMut {
             &rb
                 .new_wrapper_table::<T>()
                 .eq(column, value),
-            vec!["null", "id"],
+            &["null", "id"],
         )
             .await
     }
@@ -882,14 +896,12 @@ impl CRUD for Rbatis {
         conn.remove_batch_by_column::<T, C>(column, values).await
     }
 
-    async fn update_by_wrapper<T>(&self, table: &mut T, w: &Wrapper, update_null_value: bool) -> Result<u64> where
+    /// update_by_wrapper
+    /// skips: use &["id","null"] will skip id column and null value param
+    async fn update_by_wrapper<T>(&self, table: &mut T, w: &Wrapper, skips: &[&str]) -> Result<u64> where
         T: CRUDTable {
         let mut conn = self.acquire().await?;
-        if update_null_value {
-            conn.update_by_wrapper(table, w, vec!["null"]).await
-        } else {
-            conn.update_by_wrapper(table, w, vec![]).await
-        }
+        conn.update_by_wrapper(table, w, skips).await
     }
 
     async fn update_by_column<T>(&self, column: &str, table: &mut T) -> Result<u64> where
